/*
 * LCDW[(SC1602) 4bit[hCu[  Version 1.00
 * File:        lcd.c
 * Target:      PIC18F14K50 48Mhz
 * RpC:   Microchip MPLAB XC8 C Compiler V1.20
 * :     PICKit 3
 * [X:     2013/09/29 15:00
 *
*/

#include <p18f14k50.h>
#include <plib\delays.h>
#include "lcd.h"

//  LCDt

void lcd_init(void){
    Delay1KTCYx(180);  //  15ms*(48MHz/4)=180,000

    LCD_RS=0;
    lcd_out4( LCD_FUNCM | LCD_8BIT ); //  8bit
    Delay1KTCYx( 60);  //   5ms*(48MHz/4)= 60,000

    lcd_out4( LCD_FUNCM | LCD_8BIT ); //  8bit
    Delay1KTCYx( 48);  //   4ms*(48MHz/4)= 48,000

    lcd_out4(  LCD_FUNCM | LCD_8BIT ); //  8bit
    Delay1KTCYx( 48);  //   4ms*(48MHz/4)= 48,000

    lcd_out4( LCD_FUNCM | LCD_4BIT); //  4bit
    Delay1KTCYx( 48);  //   4ms*(48MHz/4)= 48,000

    lcd_out8( LCD_FUNCM | LCD_4BIT | 0b1000 ); //  샂[h 4bit 1/Duty 5*7dot
    Delay1KTCYx( 48);  //   4ms*(48MHz/4)= 48,000

    lcd_out8( LCD_DISPM | LCD_DISPON | LCD_CURON | LCD_BLINKOFF ); // \on
    Delay1KTCYx( 2);  //  0.167ms*(48MHz/4)=  2,000 6th

    lcd_out8( LCD_ENTRYM | LCD_INC | LCD_NONSHIFT); // Gg[[h
    Delay1KTCYx( 2);  //  0.167ms*(48MHz/4)=  2,000

    lcd_clear();
}

//  tNA

void lcd_clear(void){
    LCD_RS=0;
    lcd_out8( LCD_CLEAR );
    Delay1KTCYx(24);  //  2ms*(48MHz/4)=  24,000
}

//  t\ݒ(uNAJ[\)

void lcd_en(unsigned char d){
    LCD_RS=0;
    lcd_out8( LCD_DISPM | d);
    Delay10TCYx(48);  //  40s*(48MHz/4)=  480
}

//  JgAhXɕ

void lcd_puts(const char* buf){
    LCD_RS=1;
    while(*buf){
        lcd_out8(*buf);
        Delay10TCYx(60);  //  50s*(48MHz/4)=  600
        ++buf;
    }
}

//  JgAhX1

void lcd_putc(unsigned char c){
    LCD_RS=1;
    lcd_out8(c);
    Delay10TCYx(60);  //  50s*(48MHz/4)=  600
}

//  \pAhXݒ

void lcd_ddram(unsigned char ads){
    LCD_RS=0;
    lcd_out8( LCD_DDADS | ads );
    Delay10TCYx(48);  //  40s*(48MHz/4)=  480
}

//  \pAhXݒ ƍsŎw

void lcd_ddramxy(unsigned char x,unsigned char y){
    unsigned char ads = 0x40*y - (2<=y ? (0x40*2-0x14) : 0)+x;
    lcd_ddram(ads);
}

//  [U[`LN^ݒ
//  ads̓LN^[R[h*8Őݒ肷

void lcd_cgram(unsigned char ads,unsigned char* buf){
    LCD_RS=0;
    lcd_out8( LCD_CGADS | ads );
    Delay1TCYx(480);  //  40s*(48MHz/4)=  480
    LCD_RS=1;
    for(int n=0;n<8;n++){
       lcd_out8( buf[n] );
        Delay10TCYx(48);  //  40s*(48MHz/4)=  480
    }
}

//  z[ʒu當tɕ\ \nɂ镡sѐ܂Ԃ\T|[g

void lcd_disp(const unsigned char* buf){
    const unsigned char* p=buf;
    unsigned char x=0;    //  ʒu
    unsigned char y=0;    //  sʒu
    LCD_RS=0;
    lcd_out8( LCD_CURHOME );
    Delay1KTCYx( 24);  //  2ms*(48MHz/4)=  24,000

    LCD_RS=1;
    while(*p){
        if(x==LCD_CMAX || *p=='\n'){
            ++y;
            lcd_ddramxy(0,y);
            Delay10TCYx(60);  //  50s*(48MHz/4)=  600
            LCD_RS=1;
            x=0;
        }
        if(*p!='\n'){
            lcd_out8(*p);
            Delay10TCYx(60);  //  50s*(48MHz/4)=  600
            ++x;
        }
        ++p;
   }
}

//  4bito

void lcd_out4(unsigned char data){
    unsigned char t1;
    LCD_ADS=(LCD_ADS & 0x0f) | (data & 0xf0);
    LCD_E=1;
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    LCD_E=0;
}

//  8bit4bitɕďo

void lcd_out8(unsigned char data){
    lcd_out4(data);
    lcd_out4(data<<4);
}
